# SPDX-License-Identifier: Apache-2.0.

__version__ = "0.8.2"

DEFAULT_TIMEOUT_MS = 5000
DEFAULT_NAMESPACE_NAME = 'default'
DEFAULT_OPENLINEAGE_URL = 'http://localhost:5000'
DEFAULT_PRODUCER = f"https://github.com/OpenLineage/OpenLineage/tree/{__version__}/client/python"
